class ComicsController < ApplicationController
  before_filter :authenticate_user!, :only => [:top, :index, :show, :play, :create, :update, :destroy]
  before_filter :authenticate_admin!, :only => [:list, :browse]

  private
  
  def treat_param comic
    comic.author_id = @author.id
  end
  
  public
  
  def top
    @new_comics = Comic.find(:all, 
      :include => :author, :conditions => ['visible > 0'], :order => 'updated_at desc', :limit => 5
    )
    @new_pictures = OriginalPicture.find(:all, 
      :include => [:artist, :license, :resource_picture], :order => 'updated_at', :limit => 5
    )

    respond_to do |format|
      format.html # index.html.erb
    end
  end
  
  # GET /comics
  # GET /comics.json
  def index
    @comics = Comic.list({}, params[:page].to_i, params[:page_size].to_i)
    respond_to do |format|
      format.html # index.html.erb
      format.json { render json: @comics.to_json(Comic.list_json_opt) }
    end
  end

  # GET /comics/1
  # GET /comics/1.json
  def show
    @comic = Comic.show(params[:id])

    respond_to do |format|
      if @comic.own?(@author) or @comic.visible > 0
        format.html # show.html.erb
        format.json { render json: @comic.to_json(Comic.show_json_opt) }
      else
        format.html { render :status => :forbidden,  :file => '/403.html'}
        format.json { head :forbidden }
      end
    end
  end

  def play
    @comic = Comic.play(params[:id])
    respond_to do |format|
      format.html # index.html.erb
      format.json {
        render :json => @comic.to_json_play
      }
      format.jsonp {
        render :json => "callback(" + @comic.to_json_play + ");"
      }
    end
  end
  
  def list
    @comics = Comic.all

    respond_to do |format|
      format.html { render layout: 'system' }# index.html.erb
      format.json { render json: @comics }
    end
  end

  def browse
    @comic = Comic.find(params[:id])

    respond_to do |format|
      format.html { render layout: 'system' } # show.html.erb
      format.json { render json: @comic }
    end
  end
  
  # GET /comics/new
  # GET /comics/new.js
  def new
    @comic = Comic.new
    @comic.supply_default
    respond_to do |format|
      format.html # new.html.erb
      format.js
    end
  end

  # GET /comics/1/edit
  # GET /comics/1.js/edit
  def edit
    @comic = Comic.find(params[:id])
    respond_to do |format|
      if @comic.own? @author
        format.html 
        format.js
      else
        format.html { render :status => :forbidden,  :file => '/403.html'}
        format.js { head :forbidden }
      end
    end
  end

  # POST /comics
  # POST /comics.json
  def create
    params[:comic].merge! author_id: @author.id
    @comic = Comic.new params[:comic]
    @comic.supply_default 

    respond_to do |format|
      if @comic.save
        format.html { redirect_to @comic, notice: 'Comic was successfully created.' }
        format.json { render json: Comic.show(@comic.id).to_json(Comic.show_json_opt), status: :created, location: @comic }
      else
        format.html { render action: "new" }
        format.json { render json: @comic.errors, status: :unprocessable_entity }
      end
    end
  end

  # PUT /comics/1
  # PUT /comics/1.json
  def update
    params[:comic].merge! author_id: @author.id
    @comic = Comic.find(params[:id])
    respond_to do |format|
      if @comic.own? @author
        if @comic.update_attributes(params[:comic])
          format.html { redirect_to @comic, notice: 'Comic was successfully updated.' }
          format.json { head :ok }
        else
          format.html { render action: "edit" }
          format.json { render json: @comic.errors, status: :unprocessable_entity }
        end
      else
        format.html { render :status => :forbidden,  :file => '/403.html'}
        format.json { head :forbidden }
      end
    end
  end

  # DELETE /comics/1
  # DELETE /comics/1.json
  def destroy
    @comic = Comic.find(params[:id])
    if @comic.own? @author
      @comic.destroy
      respond_to do |format|
        format.html { redirect_to comics_url }
        format.json { head :ok }
      end
    else
      format.html { render action: "edit" }
      format.json { render json: @comic.errors, status: :unprocessable_entity }
    end
  end
end
