class Comic < ActiveRecord::Base
  has_many :panels, :dependent => :destroy
  belongs_to :author
  
  class NotZeroValidator < ActiveModel::EachValidator
    def validate_each(record, attribute, value)
      record.errors[attribute] << (options[:message] || "is zero") if value == 0
    end
  end

  validates :title, :presence => true, :length => {:maximum => 100}
  validates :width, :presence => true, :numericality => true, :not_zero => true
  validates :height, :presence => true, :numericality => true, :not_zero => true
  validates :visible, :presence => true, :numericality => true, :inclusion => {:in => 0...4}
  validates :editable, :presence => true, :numericality => true, :inclusion => {:in => 0...4}
  
  before_save do |r|
    r.supply_default
  end
  
  def supply_default
    self.visible = 0 if self.visible.blank?
    self.editable= 0 if self.editable.blank?
  end
  
  def own? author
    return false unless author
    self.author_id == author.id
  end
  
  def disp_editable
    editable == 1 ? 'O' : 'X'
  end
  
  def disp_visible
    visible == 1 ? 'O' : 'X'
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.list opt = {}, page = 1, page_size = self.default_page_size
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page = 1 if page < 1
    opt.merge!(self.list_opt) unless opt[:include]
    opt.merge!({:conditions => ['visible > 0'], :order => 'updated_at desc', :limit => page_size, :offset => (page -1) * page_size})
    Comic.find(:all, opt)
  end
  
  def self.list_opt
    {:include => :author}
  end
  
  def self.list_json_opt
    {:include => :author}
  end
  
  def self.show cid, opt = {}
    opt.merge!(self.show_opt) unless opt[:include]
    Comic.find(cid, opt)
  end
  
  def self.show_opt
    {:include => :author}
  end
  
  def self.show_json_opt
    {:include => :author}
  end
  
  def self.play cid, opt = {}
    Comic.find(cid, include: [
      :author, 
      :panels => [
        :panel_pictures => :resource_picture, 
        :balloons => :speaches
      ]
    ], order: 'panels.t, speaches.t')
  end
  
  def to_json_play
    self.to_json( :include => {:author => {}, :panels => {:methods => :panel_element}})
  end
  
end
