class ResourcePicture < ActiveRecord::Base
  belongs_to :artist
  belongs_to :license
  has_many :panel_pictures
  belongs_to :original_picture
  
  before_destroy :destroy_with_file
  
  def destroy_with_file
    PictureIO.resource_picture_io.delete self.filename
    PictureIO.resource_picture_io.class.subdirs.each do |d|
      next if d.empty?
      PictureIO.resource_picture_io.delete(self.filename, d) if PictureIO.resource_picture_io.exist?(self.filename, d)
    end
  end
  
  def self.resize(data, dw, dh)
    Magick::Image.from_blob(data).shift.resize(dw, dh)
  end
  
  #サイズの調整（limw,limhに必ず収まるように合わせる）
  def self.fix_size_both(limw, limh, w, h)
    wr = if w > limw
      limw*100/w
    else
      100
    end
    hr = if h > limh
      limh*100/h
    else
      100
    end
    res = if wr < hr
      #幅の方が圧縮率が高い
      [w*wr/100, h*wr/100]
    else
      #高さの方が圧縮率が高い
      [w*hr/100, h*hr/100]
    end
    res
  end
  
  def dext
    self.ext.downcase
  end
  
  def filename
    "#{self.id}.#{self.dext}"
  end
  
  def mime_type
    "image/#{self.dext}"
  end
  
  def v(rimg)
    rimg.flip.to_blob
  end
  
  def h(rimg)
    rimg.flop.to_blob
  end
  
  def vh(rimg)
    rimg.flip.flop.to_blob
  end
  
  def url subdir = nil
    '/resource_pictures/' + (subdir.to_s.empty? ? '' : subdir.to_s + '/' ) + filename
  end
  
  def thumbnail(rimg)
    tw, th = ResourcePicture.fix_size_both(80, 80, rimg.columns, rimg.rows)
    ResourcePicture.resize(rimg.to_blob, tw, th).to_blob
  end
  
  def self.store(img, original_picture)
    
    resource_picture = ResourcePicture.new(
      width: original_picture.width, height: original_picture.height, 
      ext: original_picture.ext, filesize: original_picture.filesize, 
      original_picture_id: original_picture.id, artist_id: original_picture.artist_id, 
      license_id: original_picture.license_id
    )
    res = if resource_picture.save
      if resource_picture.store(img)
        true
      else
        false
      end
    else
      false
    end
    res
  end
  
  def store(img)
    res = false
    bindata = img.to_blob
#    begin
      PictureIO.resource_picture_io.put bindata, self.filename
      PictureIO.resource_picture_io.class.subdirs.each do |d|
        next if d.empty?
        PictureIO.resource_picture_io.put(self.__send__(d, img), self.filename, d)
      end
      res = true
#    rescue
#      res = false
#    end
    res
  end
  
  def restore(subdir = nil)
    PictureIO.resource_picture_io.get self.filename, subdir
  end
  
end
